/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
ldse@hendriklipka.de
*/

package de.hendriklipka.ldse.server.data.document;

import java.util.*;

import com.objectspace.jgl.HashMap;
import com.objectspace.jgl.HashMapIterator;

import de.hendriklipka.ldse.server.common.Triplet;

// Liste von Triplets...
public class TripletList implements Cloneable
{
  protected int _length=0; // Anzahl verschiedener Triplets
  protected int _count=0;  // Anzahl Triplets gesamt
  
  protected HashMap _triplets=null;
  
  public TripletList(int i)
  {
    _triplets=new HashMap(Triplet.getBinaryPredicate(),false,i,0.75f);
  }
  
  public TripletList()
  {
    _triplets=new HashMap(Triplet.getBinaryPredicate());
  }
  
  /**
    erzeugt einen Liste von Triplets aus einem uebergebenen String
    dieser sollte nur ein einzelnes Wort enthalten
  */
  public TripletList(String word)
  {
    this(11); // Array initialisieren
    word=word.toLowerCase();
    int l=word.length();
    char[] ca2=new char[3];
    try
    {
      for (int i=0;i<l-2;i++)
      {
        word.getChars(i,i+3,ca2,0);
        Triplet tr=new Triplet(ca2);
        if (tr.num!=-1)
        {
          addTriplet(tr);
        }
      }
    }
    catch (Exception ex)
    {
    }
  }
  
  public Enumeration getTriplets()
  {
    return _triplets.keys();
  }

  public void addTriplet(Triplet _triplet)
  {
    addTriplet(_triplet,1);
  }
  
  public void addTriplet(Triplet _triplet, int factor)
  {
    try
    {
      Object o=_triplets.add(_triplet,_triplet);
      if(null==o)
      {
        _length++;
        _count+=(_triplet.count*factor);
      }
      else
      {
        Triplet tr=(Triplet)o;
        tr.count+=(_triplet.count*factor);
        _count+=(_triplet.count*factor);
      }
    }
    catch (Throwable ex)
    {
    }
  }
  
  public boolean hasTriplet(Triplet _triplet)
  {
    try
    {
      Triplet tr=(Triplet)_triplets.get(_triplet);
      if (null==tr)
      {
        return false;
      }
      else
      {
        return true;
      }
    }
    catch (Throwable ex)
    {
      return false;
    }
  }
  
  public Triplet getTriplet(Triplet _triplet)
  {
    try
    {
      Triplet tr=(Triplet)_triplets.get(_triplet);
      return tr;
    }
    catch (Throwable ex)
    {
      return null;
    }
  }
  
  public HashMapIterator getIterator()
  {
    return _triplets.begin();
  }
  
  public void addList(TripletList list)
  {
    addList(list,1);
  }
  
  public void addList(TripletList list, int factor)
  {
    HashMapIterator it=list.getIterator();
    while (!it.atEnd())
    {
      Triplet tr=(Triplet)it.value();
      addTriplet(tr,factor);
      it.advance();
    }
  }
  
  public int getTripletLength()
  {
    return _length;
  }
  
  public int getTripletCount()
  {
    return _count;
  }
  
  // Testfunktion
  public static void main(String[] sa)
  {
    int sum=0;
    System.out.println("running...");
    TripletList all=new TripletList();
    long start=System.currentTimeMillis();
    for (int l=0;l<200;l++)
    {
      StringBuffer sb=new StringBuffer(60000);
      java.util.Random rand=new java.util.Random();
      for (int i=0;i<65500;i++)
      {
        // Zahl von 0 bis 47 erzeugen
        int j=(rand.nextInt()&63); 
        while (j>47)
          j=(rand.nextInt()&63); 
        char c=Triplet._triples[j];
        sb.append(c);
      }
      TripletList tl=new TripletList(sb.toString());
      all.addList(tl);
      System.out.println("tl: "+tl._length);
      sum+=tl._length;
      System.out.println("all: "+all._length);
    }
    long ende=System.currentTimeMillis();
    Enumeration e=all.getTriplets();
    while (e.hasMoreElements())
    {
      Triplet tr=(Triplet)e.nextElement();
      System.out.println(tr.num+" - "+tr.count);
    }
    System.out.println(ende-start+" ms");
    System.out.println("all: "+all._length);
    System.out.println("sum: "+sum);
  }
  
  public Object clone()
  {
    TripletList list=new TripletList();
    list._length=_length;
    list._triplets=(HashMap)_triplets.clone();
    return list;
  }
}