/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
ldse@hendriklipka.de
*/

package de.hendriklipka.ldse.server.gatherer;

import java.net.*;
import java.io.*;
import java.util.*;
import java.util.zip.*;

import HTTPClient.*;

import Acme.*; // HTMLScanner

import de.hendriklipka.ldse.common.Log;
import de.hendriklipka.ldse.common.HTTPConnectionPool;
import de.hendriklipka.ldse.server.data.URLInfo;

public class SpiderRunner extends Thread implements Acme.HtmlObserver
{
  public static int urlToDo=0;
  public static Object o=new Object();
  
  HTTPSpider _parent=null;
  URLInfo _urlInfo=null;
  String _baseUrlStr=null;
    
  // The optional authorization cookie.
  private String auth_cookie = null;
  
  protected Vector actURLs=null;
  
  InputStream s=null;
  InputStream s2=null;
  Reader r=null;
  HtmlScanner scanner=null;
  Checksum cs=null;
  
  int count=0;
  int compCount=0;
  int failCount=0;
  String currURL=null;
  String compURL=null;
  boolean stop=false;
  
  public SpiderRunner(URLInfo urlInfo, HTTPSpider parent)
  {
    Log.log("spider","info",1,"spider for "+urlInfo._url.toString());
    _urlInfo=urlInfo;
    _parent=parent;
    currURL=urlInfo._urlStr;
  }
    
  public void run()
  {
    try
    {
      synchronized(o)
      {
        urlToDo++;
      }
      doUrl(_urlInfo);
    }
    catch (Throwable ex)
    {
      Log.log("spider","error",0,"SpiderRunner.run: fatal error: "+ex);
    }
    finally
    {
      // sicherstellen, dass der SpiderRunner auch wirklich ausgetragen wird
      // besonders im Fehlerfall :)
      _parent.runnerIsDone(this);
    }
  }
  
  public void doUrl(URLInfo urlInfo)
  {
    Log.log("spider","info",1,"doing URL: "+urlInfo._urlStr);
    // backup aktuelle URL-Liste 
    currURL=urlInfo._urlStr;
    Vector oldURLs=actURLs;
    actURLs=new Vector(5,5);
    boolean recursion=false;
    boolean isError=true;
    // URL laden
    try
    {
      // Connection oeffnen
      // wir nehmen Sie aus dem Pool, damit das Keep-Alive geht
      // ausserdem spart es das neue Anlegen...
      HTTPConnection con = HTTPConnectionPool.getConnection(urlInfo._url.getHost());
      HTTPResponse   rsp = con.Get(urlInfo._url.getFile()); 
      if (rsp.getStatusCode() >= 300)
      {
          _parent.brokenLink(urlInfo._contextURL,urlInfo._url,"Received Error: "+rsp.getReasonLine()+new String(rsp.getData()));
          return;
      }
      // Laenge bestimmen
      String slen=rsp.getHeader("Content-Length");
      try
      {
        urlInfo._size=Integer.parseInt(slen);
      }
      catch (NumberFormatException ex) 
      {
        urlInfo._size=-1;
      }
      // Content-Type
      urlInfo._type=rsp.getHeader("Content-Type");
      // Last-Modified-Since
      urlInfo._lastChanged=rsp.getHeaderAsDate("Last-Modified");
      if (null==urlInfo._lastChanged)
      {
        urlInfo._lastChanged=new Date();
      }
      // wenn es HTML ist, dann muss es geparst werden
      stop=false;
      if ( urlInfo._type.equals( "text/html" ) )
      {
        // Make a scanner, and pass in the URLInfo as the clientData.
        //ToDo: Checksumme fuer alle URls berechnen, nicht nur fuer HTML
        // Frage: wozu???
        s = rsp.getInputStream();
        cs=new CRC32();
        s2=new CheckedInputStream(s,cs);
        r=new InputStreamReader(s2);
        
        scanner = new HtmlScanner(r, urlInfo._url, this, urlInfo);

        // und alles einlesen, damit alles gescannt wird...
        int len=0;
        int count=0;
        // Laenge extra berechnen und dann zuweisen...
        // weil sie nicht immer richtig im Header steht
    		while ( scanner.read() != -1 ) 
    		{
    		  len++;
    		  count++;
    		  if (stop)
    		  {
    		    Log.log("spider","threads",2,"thread for "+currURL+" found stop...");
    		    break;
    		  }
    		}
    		urlInfo._size=len;
    		scanner.close();
    		// Suchen, ob wir ein Parent finden, welches identisch ist...
    		// weil dann haben wir eine Rekursion...
    		// gestartet wird mit dem Parent der aktuellen URL
    		URLInfo tempUI=urlInfo._parent;
    		while (null!=tempUI)
    		{
          // Recursion: same size, same checksum
    		  if ((urlInfo._checksum==tempUI._checksum)&&(urlInfo._size==tempUI._size))
    		  {
    		    Log.log("spider","url",0,"**** Rekursion found...\nakt. URL  : "+urlInfo._urlStr+"\nother. URL: "+tempUI._urlStr);
    		    recursion=true;
    		    break;
    		  }
    		  tempUI=tempUI._parent;
    		}
      }
      if (null!=s)
        s.close();
      if (null!=s2)
        s2.close();
      if (null!=cs)
        urlInfo._checksum=cs.getValue();

        isError=false; // alles erledigt...
    }
    catch ( FileNotFoundException e )
    {
      _parent.brokenLink(urlInfo._contextURL,urlInfo._url,e.getMessage() );
    }
    catch ( UnknownHostException e )
    {
      _parent.brokenLink(urlInfo._contextURL,urlInfo._url,"unknown host -- " + e.getMessage() );
    }
    catch ( Throwable e )
    {
      // sicherheitshalber die Connection entfernen...
      HTTPConnectionPool.removeConnection(urlInfo._url.getHost());
      e.printStackTrace(Log.getWriter("spider","error",0));
      _parent.reportError(urlInfo._contextURL,urlInfo._url,e.toString() );
    }
    finally
    {
    	if (stop)
    	{
    		Log.log("spider","thread",2,"thread found stop...");
    	}
      // Speicher mit Sicherheit wieder freigeben, damit alle Verbindungen
      // wieder freigegeben werden
      // ansonsten gehen unter Linux die Filehandles aus :-(
      s=null;
      s2=null;
      cs=null;
  	  scanner=null;
  	  System.gc();
      // URL als fertig markieren
      if ((!recursion)&&(!isError))
      {
        _parent.urlIsDone(urlInfo);
      }
      // ausgetragen wird auf jeden Fall
      synchronized(o)
      {
        urlToDo--;
      }
    }
    if (!recursion)
    {
      // geholte URLs rekursiv holen
      int i=0;
      while (!actURLs.isEmpty())
      {
        URLInfo ui=(URLInfo)actURLs.firstElement();
        actURLs.removeElementAt(0);
        // Pruefung, ob URL geholt werden muss, wurde bereits beim addURL erledigt
        if (!_parent.hasFreeSpider(ui))
        {
          doUrl(ui);
        }
        else
        {
          // wenn ein neuer Thread gestartet wurde, wird dort urlToDo um eins erhoeht...
          // aber das stimmt ja nicht...
          synchronized(o)
          {
            urlToDo--;
          }
        }
      }
    }
    actURLs=oldURLs;
    return ;
  }
  
  // gefundene URLs zur aktuellen Liste hinzufuegen
  public void addURL( String urlStr, URL contextUrl, Object clientData )
  {
    URLInfo data=(URLInfo)clientData;
    try
    {
      // Convert to no-ref, canonical form.
      URL url=null;
      if (null==contextUrl)
      {
        url=new URL(urlStr);
      }
      else
      {
        url = Acme.Utils.plainUrl( contextUrl, urlStr );
        urlStr = url.toExternalForm();
      }
      if (0==urlStr.length())
      {
        return;
      }
      // Add it.
      if (_parent.doThisUrl(urlStr,url,data._depth+1))
      {
        URLInfo ui=new URLInfo();
        ui._url=url;
        ui._urlStr=urlStr;
        ui._contextURL=contextUrl;
        ui._depth=data._depth+1;
        ui._parent=data;
        ui._spiderID=data._spiderID;
        ui._spider=_parent;
        actURLs.addElement(ui);
        Log.log("spider","url",1,"added URL: "+urlStr);
        synchronized(o)
        {
          urlToDo++;
        }
        // Also add all parent directories up to the root.  We'll get
        // a lot of duplicates this way, but the hashtable will filter
        // them out.
        // nur noetig, wenn die obige URL auch geaddet wurde
        // ansonsten haben wir auch schon alle Oberverzeichnisse...
        String rootUrlStr =(new URL(new URL(urlStr),"/")).toExternalForm();
        while ( urlStr.length() > rootUrlStr.length() )
        {
          int lastSlash = urlStr.lastIndexOf( '/', urlStr.length() - 2 );
          urlStr = urlStr.substring( 0, lastSlash + 1 );
          url=new URL(urlStr);
          if (_parent.doThisUrl(urlStr,url,data._depth+1))
          {
            ui=new URLInfo();
            ui._url=url;
            ui._urlStr=urlStr;
            ui._contextURL=contextUrl;
            ui._depth=data._depth+1;
            ui._spiderID=data._spiderID;
            ui._parent=data;
            ui._spider=_parent;
            synchronized(o)
            {
              urlToDo++;
            }
            actURLs.addElement(ui);
            Log.log("spider","url",1,"added URL: "+urlStr);
          }
          else
          {
            // wenn ein Dir schon bekannt ist, sind es auch alle Oberverzeichnisse
            break;
          }
          ui=null;
        }
      }
    }
    catch ( Exception e )
    {
    }
  }
  
  // Callback-Methoden fuer den HTML-Scanner
  
  /// This gets called when the scanner finds an &lt;A HREF=""&gt; URL.    
  public void gotAHREF( String urlStr, URL contextUrl, Object clientData )
  {
    addURL(urlStr,contextUrl,clientData);
  }
  
  /// This gets called when the scanner finds an &lt;IMG SRC=""&gt; URL.    
  public void gotIMGSRC( String urlStr, URL contextUrl, Object clientData )    
  {
    addURL(urlStr,contextUrl,clientData);
  }
  
  /// This gets called when the scanner finds an &lt;FRAME SRC=""&gt; URL.    
  public void gotFRAMESRC( String urlStr, URL contextUrl, Object clientData )    
  {
    addURL(urlStr,contextUrl,clientData);
  }
  
  /// This gets called when the scanner finds a &lt;BASE HREF=""&gt; URL.    
  public void gotBASEHREF( String urlStr, URL contextUrl, Object clientData )    
  {
    addURL(urlStr,contextUrl,clientData);
  }
  
  /// This gets called when the scanner finds a &lt;AREA HREF=""&gt; URL.    
  public void gotAREAHREF( String urlStr, URL contextUrl, Object clientData )    
  {
    addURL(urlStr,contextUrl,clientData);
  }
  
  /// This gets called when the scanner finds a &lt;LINK HREF=""&gt; URL.    
  public void gotLINKHREF( String urlStr, URL contextUrl, Object clientData )    
  {
    addURL(urlStr,contextUrl,clientData);
  }
  
  /// This gets called when the scanner finds a &lt;BODY BACKGROUND=""&gt; URL.    
  public void gotBODYBACKGROUND( String urlStr, URL contextUrl, Object clientData )
  {
    addURL(urlStr,contextUrl,clientData);
  }
  
  protected boolean checkMalformedURL( String msg )
  {
    // Java is missing protocol handlers for many common
    // protocols.  Ignore those errors.
    if ( msg.startsWith( "unknown protocol: " ) )
    {
      String protocol = msg.substring( msg.lastIndexOf( ' ' ) + 1 );
      if ( protocol.equalsIgnoreCase( "gopher" ) ||
 	        protocol.equalsIgnoreCase( "ftp" ) ||
	        protocol.equalsIgnoreCase( "file" ) ||
	        protocol.equalsIgnoreCase( "telnet" ) ||
	        protocol.equalsIgnoreCase( "news" ) ||
	        protocol.equalsIgnoreCase( "mailto" ) ||
	        protocol.equalsIgnoreCase( "javascript" ) )
        return false;
      }
    return true;
  }
}
